=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-12-17 02:48:23 +0000
+++ bzrlib/builtins.py	2007-12-17 16:04:08 +0000
@@ -2888,11 +2888,11 @@
         from bzrlib.tag import _merge_tags_if_possible
         assert revision is None or len(revision) < 3
         # find the branch locations
-        other_loc, location = self._select_branch_location(tree, location,
+        other_loc, user_location = self._select_branch_location(tree, location,
             revision, -1)
         if revision is not None and len(revision) == 2:
-            base_loc, location = self._select_branch_location(tree, location,
-                                                              revision, 0)
+            base_loc, _unused = self._select_branch_location(tree,
+                location, revision, 0)
         else:
             base_loc = other_loc
         # Open the branches
@@ -2919,9 +2919,9 @@
         else:
             base_revision_id = None
         # Remember where we merge from
-        if ((tree.branch.get_parent() is None or remember) and
-            other_branch is not None):
-            tree.branch.set_parent(other_branch.base)
+        if ((remember or tree.branch.get_submit_branch() is None) and
+             user_location is not None):
+            tree.branch.set_submit_branch(other_branch.base)
         _merge_tags_if_possible(other_branch, tree.branch)
         merger = _mod_merge.Merger.from_revision_ids(pb, tree,
             other_revision_id, base_revision_id, other_branch, base_branch)
@@ -2932,7 +2932,7 @@
             allow_pending = True
         return merger, allow_pending
 
-    def _select_branch_location(self, tree, location, revision=None,
+    def _select_branch_location(self, tree, user_location, revision=None,
                                 index=None):
         """Select a branch location, according to possible inputs.
 
@@ -2940,34 +2940,35 @@
         ``revision`` and ``index`` must be supplied.)
 
         Otherwise, the ``location`` parameter is used.  If it is None, then the
-        ``parent`` location is used, and a note is printed.
+        ``submit`` or ``parent`` location is used, and a note is printed.
 
         :param tree: The working tree to select a branch for merging into
         :param location: The location entered by the user
         :param revision: The revision parameter to the command
         :param index: The index to use for the revision parameter.  Negative
             indices are permitted.
-        :return: (selected_location, default_location).  The default location
-            will be the user-entered location, if any, or else the remembered
-            location.
+        :return: (selected_location, user_location).  The default location
+            will be the user-entered location.
         """
         if (revision is not None and index is not None
             and revision[index] is not None):
             branch = revision[index].get_branch()
             if branch is not None:
-                return branch, location
-        location = self._get_remembered_parent(tree, location, 'Merging from')
-        return location, location
+                return branch, branch
+        if user_location is None:
+            location = self._get_remembered(tree, 'Merging from')
+        else:
+            location = user_location
+        return location, user_location
 
-    # TODO: move up to common parent; this isn't merge-specific anymore. 
-    def _get_remembered_parent(self, tree, supplied_location, verb_string):
+    def _get_remembered(self, tree, verb_string):
         """Use tree.branch's parent if none was supplied.
 
         Report if the remembered location was used.
         """
-        if supplied_location is not None:
-            return supplied_location
-        stored_location = tree.branch.get_parent()
+        stored_location = tree.branch.get_submit_branch()
+        if stored_location is None:
+            stored_location = tree.branch.get_parent()
         mutter("%s", stored_location)
         if stored_location is None:
             raise errors.BzrCommandError("No location specified or remembered")

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2007-12-04 03:13:51 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2007-12-17 16:04:08 +0000
@@ -136,7 +136,7 @@
         self.failUnlessExists('sub/a.txt.BASE')
 
     def test_merge_remember(self):
-        """Merge changes from one branch to another and test parent location."""
+        """Merge changes from one branch to another, test submit location."""
         tree_a = self.make_branch_and_tree('branch_a')
         branch_a = tree_a.branch
         self.build_tree(['branch_a/a'])
@@ -166,15 +166,18 @@
         tree_b.add('d')
         self.run_bzr_error(['Working tree ".*" has uncommitted changes'],
                            'merge ../branch_a')
-        self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
+        self.assertEquals(abspath(branch_b.get_submit_branch()),
+                          abspath(parent))
         # test implicit --remember after resolving conflict
         tree_b.commit('commit d')
         out, err = self.run_bzr('merge')
         
         base = urlutils.local_path_from_url(branch_a.base)
-        self.assertEquals(out, 'Merging from remembered location %s\n' % (base,))
+        self.assertEquals(out,
+                          'Merging from remembered location %s\n' % (base,))
         self.assertEquals(err, '+N  b\nAll changes applied successfully.\n')
-        self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
+        self.assertEquals(abspath(branch_b.get_submit_branch()),
+                          abspath(parent))
         # re-open tree as external run_bzr modified it
         tree_b = branch_b.bzrdir.open_workingtree()
         tree_b.commit('merge branch_a')
@@ -182,7 +185,7 @@
         out, err = self.run_bzr('merge ../branch_c --remember')
         self.assertEquals(out, '')
         self.assertEquals(err, '+N  c\nAll changes applied successfully.\n')
-        self.assertEquals(abspath(branch_b.get_parent()),
+        self.assertEquals(abspath(branch_b.get_submit_branch()),
                           abspath(branch_c.bzrdir.root_transport.base))
         # re-open tree as external run_bzr modified it
         tree_b = branch_b.bzrdir.open_workingtree()
@@ -406,3 +409,27 @@
         graph = tree_a.branch.repository.get_graph(tree_b.branch.repository)
         out, err = self.run_bzr(['merge', '-d', 'a', 'b'])
         self.assertContainsRe(err, 'Warning: criss-cross merge encountered.')
+
+    def test_merge_from_submit(self):
+        tree_a = self.make_branch_and_tree('a')
+        tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
+        tree_c = tree_a.bzrdir.sprout('c').open_workingtree()
+        out, err = self.run_bzr(['merge', '-d', 'c'])
+        self.assertContainsRe(out, 'Merging from remembered location .*a\/$')
+        tree_c.branch.set_submit_branch(tree_b.bzrdir.root_transport.base)
+        out, err = self.run_bzr(['merge', '-d', 'c'])
+        self.assertContainsRe(out, 'Merging from remembered location .*b\/$')
+
+    def test_remember_sets_submit(self):
+        tree_a = self.make_branch_and_tree('a')
+        tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
+        self.assertIs(tree_b.branch.get_submit_branch(), None)
+
+        # Remember should not happen if using default from parent
+        out, err = self.run_bzr(['merge', '-d', 'b'])
+        self.assertIs(tree_b.branch.get_submit_branch(), None)
+
+        # Remember should happen if user supplies location
+        out, err = self.run_bzr(['merge', '-d', 'b', 'a'])
+        self.assertEqual(tree_b.branch.get_submit_branch(),
+                         tree_a.bzrdir.root_transport.base)

